/*
 This file is part of the digitalSTROM ds485-client library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-client library is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 digitalSTROM ds485-client library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with digitalSTROM ds485-client library. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DS485CLIENTINTERFACE_H_
#define DS485CLIENTINTERFACE_H_

#include <digitalSTROM/ds.h>
#include <digitalSTROM/ds485.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef void *ds485ClientHandle_t;

enum {
    ERROR_RESPONSE_TIMEOUT = -1,
    NOTICE_CALL_TO_BROADCAST = -2,
    ERROR_INVALID_HANDLE = -3,
    ERROR_INVALID_CONNSPEC = -4,
    ERROR_THREAD_START_FAILED = -5,
    ERROR_NO_PORT_FOUND = -6,
    ERROR_INVALID_ADDRESS = -7,
    ERROR_CREATE_SOCKET = -8,
    ERROR_SOCKET_OPTIONS = -9,
    ERROR_SOCKET_CONNECT = -10,
    ERROR_SOCKET_FULL = -11,
    ERROR_SOCKET_FAILED = -12,
    ERROR_SOCKET_VANISHED = -13,
    ERROR_SOCKET_WOULD_BLOCK = -14,
    ERROR_SOCKET_UNKNOWN = -15,
    ERROR_INVALID_FD = -16,
    ERROR_INVALID_PARAMETER = -17,
    ERROR_REC_WRONG_FUNCTION_ID = -18,
    ERROR_REC_WRONG_MODIFIER = -19,
    ERROR_SYNC_RESPONSE_TIMEOUT = -20,
};

/**
 * Callback Methoden
 */
typedef int (*client_callback_state_t)(void *data, const bus_state_t state);
typedef int (*client_callback_bus_change_t)(void *data, dsid_t *id, int flags);
typedef int (*client_callback_container_t)(void *data, const ds485_container_t *container);

enum client_open_flags {
    PROMISCUOUS_MODE = 0x1, /**< Return all RESPONSE-type messages instead of only the ones in answer to own REQUESTs */
};

/**
 * Opens a connection using the specified connection type
 *
 * @CONNSPEC:
 *
 *   serial:/dev/ttyUSB0 (GPL version only)
 *   tcp:localhost:8442
 *   udp:10.0.230.87:8442
 *   unix:/tmp/ds485.socket
 *
 * @RETURN:
 *   negative value on failure
 *   positive value is the client index
 */
extern ds485ClientHandle_t ds485_client_open(const char *connspec, int flags);

/**
 * Closes the session, frees all connection related ressources
 */
extern int ds485_client_close(ds485ClientHandle_t handle);

/**
 * Query the list of devices on the dS485 bus
 *
 * @param client_index
 * @param device_list
 * @param device_list_size
 * @return negative if error; number of found devices otherwise
 */
extern int ds485_client_query_devices(ds485ClientHandle_t handle, dsid_t *device_list, int device_list_size);

/**
 * Query the current bus state
 */
extern int ds485_client_query_state(ds485ClientHandle_t handle, bus_state_t *state);

/**
 * Restart the dS485 bus.
 */
extern int ds485_client_restart_bus(ds485ClientHandle_t handle);

/**
 * Send dM-API command
 */
extern int ds485_client_send_command(ds485ClientHandle_t handle, ds485_container_t *request);

/**
 * Send dM-API command, receive a single response packet
 *
 * @return 0: success; >0: error from device; -1: failure; -2: request to broadcast will return responses by callback
 */
extern int ds485_client_send_sync_command(ds485ClientHandle_t handle, ds485_container_t *request, ds485_container_t *response, int timeout);

/**
 * Set and remove a reception handler for dS485 Packet's, optional a filter for a message id can be given
 */
extern client_callback_container_t ds485_client_set_receive_callback(ds485ClientHandle_t handle, client_callback_container_t cb, void *data);

/**
 * Set and remove a callback handler to register for bus state changes
 */
extern client_callback_state_t ds485_client_set_state_callback(ds485ClientHandle_t handle, client_callback_state_t cb, void *data);

/**
 * Set and remove a callback handler to register for bus device registration/deregistration changes
 */
extern client_callback_bus_change_t ds485_client_set_bus_change_callback(ds485ClientHandle_t handle, client_callback_bus_change_t cb, void *data);

/**
 * Get dSID of library.
 *
 * @param handle [in] Library handle
 * @param dsid [out] own dsid
 * @return 0 on success; negative on failure
 */
extern int ds485_client_get_dsid(ds485ClientHandle_t handle, dsid_t *dsid);
#ifdef __cplusplus
}
#endif

#endif /* DS485CLIENTINTERFACE_H_ */
