/*
 This file is part of the digitalSTROM ds485-client library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-client library is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 digitalSTROM ds485-client library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with digitalSTROM ds485-client library. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <getopt.h>
#include <termios.h> /* POSIX terminal control definitions */
#include <pthread.h>
#include <fcntl.h>   /* File control definitions */
#include <sys/unistd.h>
#include <stdlib.h>

#include "ds485-client-interface.h"

void formatContainer(char *pTarget, const ds485_container_t *pMessage)
{
    char *pChar = pTarget;
    unsigned char n;

    pChar += sprintf(pChar, "  Type :   %.1x  ", pMessage->containerType);
    pChar += sprintf(pChar, " msgId :   %.2x\n", pMessage->data[0]);
    pChar += sprintf(pChar, "  from :   ");
    for (n = 0; n < 12; n++)
        pChar += sprintf(pChar, "%02x", pMessage->sourceId.id[n]);
    pChar += sprintf(pChar, "\n");
    pChar += sprintf(pChar, "    to :    ");
    for (n = 0; n < 12; n++)
        pChar += sprintf(pChar, "%02x", pMessage->destinationId.id[n]);
    pChar += sprintf(pChar, "\n");
    pChar += sprintf(pChar, "param1 : %.2x%.2x ", pMessage->data[2], pMessage->data[1]);
    pChar += sprintf(pChar, "param2 : %.2x%.2x ", pMessage->data[4], pMessage->data[3]);
    pChar += sprintf(pChar, "param3 : %.2x%.2x ", pMessage->data[6], pMessage->data[5]);
    pChar += sprintf(pChar, "param4 : %.2x%.2x\r\n", pMessage->data[8], pMessage->data[7]);
    pChar += sprintf(pChar, "param5 : %.2x%.2x ", pMessage->data[10], pMessage->data[9]);
    pChar += sprintf(pChar, "param6 : %.2x%.2x ", pMessage->data[12], pMessage->data[11]);
    pChar += sprintf(pChar, "param7 : %.2x%.2x ", pMessage->data[14], pMessage->data[13]);
    pChar += sprintf(pChar, "length : %d\r\n", pMessage->length);
}

int client_callback_state(void *data, const bus_state_t state)
{
    printf("callback state: %d\n", state);
    return 0;
}
int client_callback_container(void *data, const ds485_container_t *container)
{
    char text[1000];
    formatContainer(text, container);
    printf("callback container: %s\n", text);
    return 0;
}

/* Command line parsing */
extern char *optarg;
extern int optind;

static int verbose = 1;
static int mainActive = 1;
static int serialLoopback;
static int rxFilterFlags;

/* Serial Device Name of the device we manage */
static char* connspec;
static const char conndefault[] = "serial:/dev/ttyUSB0";

int main(int argc, char *argv[])
{
    int result;
    char c;
    ds485ClientHandle_t handle;

    connspec = (char *) conndefault;

    /* Look for options */
    while ((c = getopt(argc, argv, "hVv:f:b:L")) != -1) {
        switch (c)
        {
        case 'v':
            verbose = atoi(optarg);
            printf("Set debug level to: 0x%02x\n", verbose);
            break;
        case 'f':
            rxFilterFlags = strtoul(optarg, NULL, 0);
            printf("Set rx filter to: 0x%02x\n", rxFilterFlags);
            break;
        case 'L':
            serialLoopback = 1;
            printf("Enable loopback transmission\n");
            break;
        default:
            exit(-1);
        }
    }

    /* Last arg is device or connection name. */
    if (*optarg < argc)
        connspec = strdup(argv[argc - 1]);

    handle = ds485_client_open(connspec, 0);
    if (handle == 0) {
        printf("ds485_client_open failed\n");
        return -1;
    }

    ds485_client_set_receive_callback(handle, client_callback_container, NULL);

    ds485_client_set_state_callback(handle, client_callback_state, NULL);

    ds485_container_t pFrame;
    pFrame.containerType = DS485_CONTAINER_REQUEST;
    pFrame.data[0] = 0x90;
    pFrame.length = 1;
    SetBroadcastId(pFrame.destinationId);
    ds485_client_send_command(handle, &pFrame);

    ds485_container_t pFrame2;
    pFrame2.containerType = DS485_CONTAINER_REQUEST;
    pFrame2.data[0] = 0x91;
    pFrame2.length = 1;
    SetBroadcastId(pFrame2.destinationId);
    ds485_client_send_command(handle, &pFrame2);

#if 0
    {
        uint16_t ZoneId;
        uint8_t NumberOfGroups;
        uint8_t Name[20];
        result = ZoneInfoOfIndex_Get(dsid, 0, &ZoneId, &NumberOfGroups, Name);
        printf("ZoneId: %d\nResult: %x\n", ZoneId, result);
        uint16_t DeviceCount;
        result = ZoneDeviceCount_OnlyActive(dsid, ZoneId, &DeviceCount);
        printf("DeviceCount: %d\nResult: %x\n", DeviceCount, result);
    }

    {
        int i;
        uint32_t RevHW;
        uint32_t RevSW;
        dsid_t DSID;
        uint8_t Name[20];
        result = MeterInfo_Get(dsid, &RevHW, &RevSW, &DSID, Name);

        printf("HW Revision: 0x%x\nSW Revision: 0x%x\ndSID: ", RevHW, RevSW);
        for (i = 0; i < 12; ++i)
        {
            printf("%02x ", DSID[i]);
        }
        printf("\nName: %s\nResult: %x\n", Name, result);
    }
#endif

    while (mainActive) {
        int key = getchar();
        switch (key)
        {
        case '?':
        case 'h':
            break;
        case 'q':
            mainActive = 0;
            break;
        }
    }

    result = ds485_client_close(handle);
    if (result < 0) {
        printf("ds485_client_close failed [%d]\n", result);
        return -1;
    }

    return 0;
}
